﻿using System;
using System.Linq;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Collections.Generic;
using ASPItalia.Pubs.Common.ObjectModel;

namespace ASPItalia.Pubs.Data
{
    internal sealed class BookData
    {
        private readonly PubsDataContext _context;

        public BookData(PubsDataContext context)
        {
            if (context == null)
                throw new ArgumentNullException("context");

            _context = context;
        }

        public List<Book> GetBookList()
        {
            DataLoadOptions options = new DataLoadOptions();
            options.LoadWith<Book>(b => b.Authors);
            options.LoadWith<Book>(b => b.Publisher);
            options.LoadWith<BookAuthor>(ba => ba.Author);

            _context.LoadOptions = options;

            var books =
                (from b in _context.Books
                 orderby b.Title
                 select b)
                .ToList<Book>();

            return books;
        }

        public Book GetBook(string bookID)
        {
            if (string.IsNullOrEmpty(bookID))
                return new Book();

            DataLoadOptions options = new DataLoadOptions();
            options.LoadWith<Book>(b => b.Authors);
            options.LoadWith<Book>(b => b.Publisher);
            options.LoadWith<BookAuthor>(ba => ba.Author);

            _context.LoadOptions = options;

            var book = _context.Books.SingleOrDefault<Book>(b => b.BookID.Equals(bookID));
            return book ?? new Book();
        }

        public void CreateBook(Book book)
        {
            if (book == null)
                throw new ArgumentNullException("book");

            if ((!string.IsNullOrEmpty(book.BookID)) && ((book.Authors != null) && (book.Authors.Count > 0)) && ((book.Publisher != null) && (!string.IsNullOrEmpty(book.Publisher.PublisherID))))
                _context.Books.InsertOnSubmit(book);
        }

        public void UpdateBook(Book book)
        {
            if (book == null)
                throw new ArgumentNullException("book");

            if ((!string.IsNullOrEmpty(book.BookID)) && ((book.Authors != null) && (book.Authors.Count > 0)) && ((book.Publisher != null) && (!string.IsNullOrEmpty(book.Publisher.PublisherID))))
            {
                Book b = GetBook(book.BookID);
                if (!string.IsNullOrEmpty(b.BookID))
                {
                    b.Category = book.Category;
                    b.Price = book.Price;
                    b.PublicationDate = book.PublicationDate;
                    b.Title = book.Title;
                }

                Publisher p = new PublisherData(_context).GetPublisher(book.PublisherID);
                if (!string.IsNullOrEmpty(p.PublisherID))
                {
                    b.PublisherID = p.PublisherID;
                    b.Publisher = p;
                }

                foreach(var item in b.Authors)
                    _context.BookAuthors.DeleteOnSubmit(item);

                foreach (var item in book.Authors)
                    _context.BookAuthors.InsertOnSubmit(new BookAuthor() { BookID = item.BookID, AuthorID = item.AuthorID });
            }
        }

        public void DeleteBook(string bookID)
        {
            if (string.IsNullOrEmpty(bookID))
                throw new ArgumentException("Invalid book ID");

            IEnumerable<BookAuthor> bookAuthors = _context.BookAuthors.Where<BookAuthor>(ba => ba.Book.BookID.Equals(bookID));
            Book book = GetBook(bookID);

            if (bookAuthors != null)
                foreach(var item in bookAuthors)
                    _context.BookAuthors.DeleteOnSubmit(item);

            if (book != null)
                _context.Books.DeleteOnSubmit(book);
        }
    }
}
